-- Copyright 2023 by Todd Hundersmarck (ThundR)
-- All Rights Reserved

THCore = {
MSG = {
ARGUMENT_INVALID = "Invalid argument %q (%s) in function call"
}
}
local THCore_mt = Class(THCore)
function THCore:new(dataKey, mt)
mt = Utils.getNoNil(mt, THCore_mt)
if type(dataKey) ~= "string" or type(mt) ~= "table" then
printf("ERROR (%s): Invalid call to THCore.new()", g_currentModName)
return
end
local newEnv = setmetatable({}, mt)
if newEnv then
local modInfo = g_modManager:getModByName(g_currentModName)
local missionInfo = g_mpLoadingScreen.missionInfo
local missionDynamicInfo = g_mpLoadingScreen.missionDynamicInfo
newEnv.isServer = g_server ~= nil
newEnv.isClient = g_client ~= nil
newEnv.dataKey  = dataKey
newEnv.actionEventIds = {}
newEnv.isMultiplayer  = missionDynamicInfo ~= nil and missionDynamicInfo.isMultiplayer
newEnv.isEnabled      = true
newEnv.modInfo  = modInfo
newEnv.modName  = modInfo.modName
newEnv.modTitle = modInfo.title
newEnv.modPath  = modInfo.modDir
newEnv.modList  = missionDynamicInfo.mods
newEnv.loadedMods = {
byId    = {},
byIndex = {}
}
local mapInfo = missionInfo.map
local mapIsMod = Utils.getNoNil(mapInfo.isModMap, false)
local mapModName = nil
local mapModPath = mapInfo.baseDirectory
if mapIsMod then
mapModName = g_mapManager:getModNameFromMapId(mapInfo.id)
if Utils.getNoNil(mapModName, "") == ""
or Utils.getNoNil(mapModPath, "") == ""
then
mapIsMod = false
mapModName = nil
mapModPath = nil
end
end
newEnv.mapInfo  = mapInfo
newEnv.mapIsMod = mapIsMod
newEnv.mapModName = mapModName
newEnv.mapModPath = mapModPath
newEnv.settingsModName = "FS22_TH_ModSettings"
newEnv.modSettingsPath = g_currentModSettingsDirectory
newEnv.settings = {
byId      = {},
byIndex   = {},
idToIndex = {}
}
newEnv.SETTING = newEnv.settings.idToIndex
end
newEnv:initializeLoadedMods()
return newEnv
end
function THCore:initializeLoadedMods()
local modList  = self.modList
for _, modInfo in ipairs(modList) do
local modName = Utils.getNoNil(modInfo.modName, "")
if modName ~= "" then
local modEntry = {
id   = modName:upper(),
info = modInfo,
name = modName,
path = modInfo.modDir,
file = modInfo.modFile
}
table.insert(self.loadedMods.byIndex, modEntry)
modEntry.index = #self.loadedMods.byIndex + 1
self.loadedMods.byId[modEntry.id] = modEntry
if modName == self.mapModName then
self.mapModInfo = modInfo
self.mapModFile = modInfo.modFile
end
if modName == self.settingsModName then
self.settingsModInfo = modInfo
self.settingsModPath = modInfo.modDir
self.settingsModFile = modInfo.modFile
end
end
end
end
function THCore:displayMsg(text, ...)
if type(text) ~= "string" then
text = string.format("ERROR: Invalid displayMsg text, %q", text)
end
local msgString = string.format(">> %s: ", self.modName)..string.format(text, ...)
print(msgString)
end
function THCore:warningMsg(text, ...)
if type(text) == "string" then
text = "WARNING: "..text
end
self:displayMsg(text, ...)
end
function THCore:errorMsg(raise, text, ...)
raise = Utils.getNoNil(raise, true)
if raise then
if raise == true then
raise = 2
end
if type(raise) ~= "number" then
self:displayMsg("ERROR: "..THCore.MSG.ARGUMENT_INVALID, "raise", raise)
else
if type(text) == "string" then
text = string.format(text, ...)
end
error(text, raise)
end
else
if type(text) == "string" then
text = "ERROR: "..text
end
self:displayMsg(text, ...)
printCallstack()
end
end
function THCore:assert(expression, raise, text, ...)
if not expression then
if raise == false then
self:warningMsg(text, ...)
else
if raise == true then raise = 3 end
self:errorMsg(raise, text, ...)
end
end
return expression
end
function THCore:msgOnTrue(expression, raise, text, ...)
if expression then
if raise == false then
self:warningMsg(text, ...)
else
if raise == true then raise = 3 end
self:errorMsg(raise, text, ...)
end
return true
end
return false
end
function THCore:argIsInvalid(expression, name, arg, raise)
raise = Utils.getNoNil(raise, true)
if raise == true then raise = 3 end
if not expression then
self:errorMsg(raise, THCore.MSG.ARGUMENT_INVALID, name, arg)
return true
end
return false
end
function THCore:argIsValid(expression, name, arg, raise)
raise = Utils.getNoNil(raise, true)
if raise == true then raise = 3 end
if not expression then
self:errorMsg(raise, THCore.MSG.ARGUMENT_INVALID, name, arg)
return false
end
return true
end
function THCore:pack(...)
return {n = select("#", ...), ...}
end
function THCore:unpack(target, index)
if  self:argIsValid(type(target) == "table", "target", target)
and (index == nil or self:argIsValid(type(index) == "number", "index", index))
then
local numEntries = Utils.getNoNil(target.n, #target)
index = Utils.getNoNil(index, 1)
if index <= numEntries then
return unpack(target, index, numEntries)
end
end
end
function THCore:pcall(target, a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p,q,r,s,t)
local function errHandler(pErrMsg)
if type(pErrMsg) == "string" then
self:displayMsg("ERROR: "..pErrMsg)
end
self.isEnabled = false
printCallstack()
end
local function protectedFunc()
local vTargetType = type(target)
local vFunc = target
if vTargetType == "function" then
return vFunc(a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p,q,r,s,t)
elseif vTargetType == "string" then
vFunc = self[target]
return vFunc(self, a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p,q,r,s,t)
elseif vTargetType == "table" then
vFunc = target[a]
return vFunc(target, b,c,d,e,f,g,h,i,j,k,l,m,n,o,p,q,r,s,t)
else
local vErrMsg = string.format(THCore.MSG.ARGUMENT_INVALID, "target", target)
error(vErrMsg, 3)
end
end
return xpcall(protectedFunc, errHandler)
end
function THCore:call(...)
local function callFunc(pSuccess, ...)
if pSuccess then
return ...
end
end
return callFunc(self:pcall(...))
end
function THCore:toNumber(value)
local valType = type(value)
if value == nil or valType == "number" then
return value
end
return tonumber(value)
end
function THCore:toBoolean(value)
local valType = type(value)
if value == nil or valType == "boolean" then
return value
end
if valType == "string" then
local valString = value:upper()
if valString == "TRUE" then
return true
elseif valString == "FALSE" then
return false
end
end
end
function THCore:getIsType(value, typeList)
local valType = type(value)
local listType = type(typeList)
if valType == typeList then
return true
end
if listType == "table" then
if valType == "table" and value.isa and value:isa(typeList) then
return true
end
elseif listType == "string" then
if string.find(typeList, "^"..valType.."|") ~= nil
or string.find(typeList, "|"..valType.."$") ~= nil
or string.find(typeList, "|"..valType.."|") ~= nil
then
return true
end
else
self:argIsInvalid(nil, "typeList", typeList)
end
return false
end
function THCore:getGlobalEnv()
local mt = getmetatable(_G)
if mt then
return Utils.getNoNil(mt.__index, _G)
end
return _G
end
function THCore:properCase(text, removeSpaces)
if  (text == nil or self:argIsValid(type(text) == "string", "text", text))
and (not removeSpaces or self:argIsValid(removeSpaces == true, "removeSpaces", removeSpaces))
then
if text == nil then return end
local retText = text
retText = string.gsub(retText, "^%l", string.upper)
retText = string.gsub(retText, " %l", string.upper)
if removeSpaces then
retText = string.gsub(retText, " ", "")
end
return retText
end
end
function THCore:getFilename(fileName, filePath, getFullName)
if fileName == nil then return end
local customEnv = nil
if  self:argIsValid(type(fileName) == "string", "fileName", fileName)
and (filePath == nil or self:argIsValid(type(filePath) == "string", "filePath", filePath))
and (not getFullName or self:argIsValid(getFullName == true, "getFullName", getFullName))
then
if fileName:sub(1,1) == "$" then
fileName = fileName:sub(2)
local modName = nil
local sPos, ePos = fileName:find("^.*%$")
if sPos and sPos > 0 then
modName  = fileName:sub(1, ePos - 1)
fileName = fileName:sub(ePos + 1)
local modInfo = self:getLoadedMod(modName)
if modInfo then
if fileName:sub(1,1) == "/" then
fileName = fileName:sub(2)
end
filePath  = modInfo.modDir
customEnv = modInfo.modName
else
modName = modName:lower()
if modName == "mapdir" then
if self.mapIsMod and self.mapModName and self.mapModPath then
filePath = self.mapModPath
customEnv = self.mapModName
end
end
end
else
filePath = getAppBasePath()
end
else
customEnv = self.modName
end
local retFilename = fileName
if getFullName then
retFilename = Utils.getFilename(fileName, filePath)
end
return retFilename, customEnv
end
end
function THCore:printTable(...)
if g_thDebugger then
g_thDebugger:printTable(...)
end
end
function THCore:clearTable(target)
if self:argIsValid(type(target) == "table", "target", target) then
for key in pairs(target) do
target[key] = nil
end
return target
end
end
function THCore:createDataTable(target, clear)
if  self:argIsValid(type(target) == "table", "target", target)
and (not clear or self:argIsValid(clear == true, "clear", clear))
then
local dataKey = self.dataKey
local dataTable = target[dataKey]
if not dataTable then
dataTable = {}
dataTable.target = target
target[dataKey] = dataTable
else
if clear then
for key in pairs(dataTable) do
dataTable[key] = nil
end
dataTable.target = target
end
end
return dataTable, target
end
end
function THCore:createOverlayTable(target)
if self:argIsValid(type(target) == "table", "target", target) then
local function callFunc(pSelf, pCommand, ...)
if pCommand == "getTarget" then
return target
elseif pCommand == "setValue" then
return rawset(pSelf, ...)
end
self:errorMsg(3, "Invalid command %q sent to overlay table", pCommand)
end
local overlayTable = setmetatable({}, {__index = target, __newindex = target, __call = callFunc})
return overlayTable, target
end
end
function THCore:getDataTable(target, create)
if target == nil and not create then
return
end
if self:argIsValid(type(target) == "table", "target", target) then
if not create then
return target[self.dataKey], target
elseif self:argIsValid(create == true, "create", create) then
return self:createDataTable(target)
end
end
end
function THCore:getTableLength(target)
if self:argIsValid(type(target) == "table", "target", target) then
local numEntries = 0
for _ in pairs(target) do
numEntries = numEntries + 1
end
return numEntries
end
return 0
end
function THCore:getTableIsArray(target)
if self:argIsValid(type(target) == "table", "target", target) then
local numEntries = 1
for _, val in pairs(target) do
if type(val) == "boolean" or target[numEntries] == nil then
return false
end
numEntries = numEntries + 1
end
return true
end
return false
end
function THCore:hookFunction(srcClass, srcFuncName, tgtClass, tgtFuncName, useSelf)
useSelf = Utils.getNoNil(useSelf, true)
local srcClassName = srcClass
if type(srcClassName) == "string" then
local globalEnv = self:getGlobalEnv()
srcClass = globalEnv[srcClassName]
end
if self:argIsInvalid(type(srcClass) == "table", "srcClass", srcClassName)
or self:argIsInvalid(type(srcFuncName) == "string", "srcFuncName", srcFuncName)
or self:argIsInvalid(type(tgtClass) == "table", "tgtClass", tgtClass)
or self:argIsInvalid(tgtFuncName == nil or type(tgtFuncName) == "sting", "tgtFuncName", tgtFuncName)
or self:argIsInvalid(type(useSelf) == "boolean", "useSelf", useSelf)
then
return false
end
tgtFuncName = Utils.getNoNil(tgtFuncName, "hook_"..srcFuncName)
local srcFunc = srcClass[srcFuncName]
local tgtFunc = tgtClass[tgtFuncName]
if srcFunc == nil then
srcFunc = function(...) end
end
if  self:assert(type(srcFunc) == "function", nil, "Invalid source function %q", srcFuncName)
and self:assert(type(tgtFunc) == "function", nil, "Invalid target function %q", tgtFuncName)
then
srcClass[srcFuncName] = function(p1, ...)
if self.isEnabled then
if useSelf then
return tgtFunc(p1, srcFunc, ...)
end
return tgtFunc(srcFunc, p1, ...)
end
return srcFunc(p1, ...)
end
return true
end
return false
end
function THCore:getFunctionData(target, funcName, recursive)
local targetName = target
if type(targetName) == "string" then
local globalEnv = self:getGlobalEnv()
target = globalEnv[targetName]
end
if  self:argIsValid(type(target) == "table", "target", targetName)
and self:argIsValid(type(funcName) == "string", "funcName", funcName)
and (not recursive or self:argIsValid(recursive == true, "recursive", recursive))
then
local trueTable = target
local superFunc = trueTable[funcName]
local origFunc  = rawget(trueTable, funcName)
if type(superFunc) == "function" then
if not recursive or origFunc == superFunc then
return trueTable, superFunc, origFunc
end
if type(trueTable.class) == "function" then
trueTable = trueTable:class()
end
while type(trueTable) == "table" do
origFunc = rawget(trueTable, funcName)
if origFunc == superFunc then
return trueTable, superFunc, origFunc
end
if type(trueTable.superClass) ~= "function" then
break
end
trueTable = trueTable:superClass()
end
end
end
end
function THCore:getLoadedMod(modName, verbose, raise)
if not verbose or self:argIsValid(verbose == true, "verbose", verbose) then
local valType = type(modName)
local modInfo = nil
if valType == "string" then
modInfo = self.loadedMods[modName:upper()]
elseif modName ~= nil then
verbose = true
end
raise = Utils.getNoNil(raise, verbose)
self:msgOnTrue(modInfo == nil and verbose, raise, THCore.MSG.ARGUMENT_INVALID, "modName", modName)
return modInfo
end
end
function THCore:getLoadedModList()
return self.loadedMods.byIndex
end
function THCore:getSettingsModFilename(filename)
if self:argIsValid(type(filename) == "string", "filename", filename) then
local settingsModInfo = self.settingsModInfo
if settingsModInfo then
local baseDir = self.settingsModPath.."mods/"..self.modName.."/"
local retFilename = nil
if self.mapIsMod and self.mapModName then
local newBaseDir = baseDir..self.mapModName.."/"
retFilename = newBaseDir..filename
if fileExists(retFilename) then
return retFilename, newBaseDir
end
end
retFilename = baseDir..filename
if fileExists(retFilename) then
return retFilename, baseDir
end
end
end
end
function THCore:getElementByProfileName(baseElement, profileName, recursive)
if  self:argIsValid(type(baseElement) == "table", "baseElement", baseElement)
and self:argIsValid(type(profileName) == "string", "profileName", profileName)
and (not recursive or self:argIsValid(recursive == true, "recursive", recursive))
then
if baseElement.profile == profileName then
return baseElement
end
local elementList = baseElement.elements
if elementList then
local numElements = #elementList
if numElements > 0 then
for i = 1, #elementList do
local element = elementList[i]
if recursive then
local foundElement = self:getElementByProfileName(element, profileName, recursive)
if foundElement then
return foundElement
end
elseif element.profile == profileName then
return element
end
end
end
end
end
end