-- Copyright 2023 by Todd Hundersmarck (ThundR)
-- All Rights Reserved

THPlaceableColorable = {
MAX_NUM_COLOR_SCALES = 8
}
function THPlaceableColorable.setAllColors(self, specData)
if  g_thMain:argIsValid(type(self) == "table", "self", self)
and g_thMain:argIsValid(type(specData) == "table", "specData", specData)
then
if specData.hasExtendedColors then
for i = 1, THPlaceableColorable.MAX_NUM_COLOR_SCALES do
local colorsList = specData.colors[i]
local colorIndex = specData.colorIndex[i]
local colorValue = colorsList[colorIndex]
if colorValue then
local shaderParam = string.format("colorScale%d", i-1)
local r,g,b = g_thMain:unpack(colorValue)
for _, nodeId in pairs(specData.nodes[i]) do
setShaderParameter(nodeId, shaderParam, r,g,b, 1, false)
end
end
end
end
end
end
function THPlaceableColorable.getAvailableColorsAtIndex(self, index, specData)
if  g_thMain:argIsValid(type(self) == "table", "self", self)
and g_thMain:argIsValid(type(index) == "number", "index", index)
and g_thMain:argIsValid(type(specData) == "table", "specData", specData)
then
if specData.hasExtendedColors then
return specData.colors[index]
end
return self:getAvailableColors()
end
end
function THPlaceableColorable.hook_registerXMLPaths(superFunc, schema, baseKey, ...)
local ra,rb,rc,rd,re = superFunc(schema, baseKey, ...)
local function appendFunc()
schema:setXMLSpecializationType("Colorable")
schema:register(XMLValueType.COLOR, baseKey..".placeableColors.sharedColors.color(?)#value", "Shared RGB color value")
for i = 1, THPlaceableColorable.MAX_NUM_COLOR_SCALES do
local csKey = string.format("%s.placeableColors.colorScale%d", baseKey, i-1)
local nodesKey = csKey..".nodes"
local colorsKey = csKey..".colors"
schema:register(XMLValueType.NODE_INDEX, nodesKey..".node(?)#index", "Node index")
schema:register(XMLValueType.BOOL,  colorsKey.."#useShared", "Use shared colors", true)
schema:register(XMLValueType.COLOR, colorsKey..".color(?)#value", "RGB color value")
end
schema:setXMLSpecializationType()
end
g_thMain:pcall(appendFunc)
return ra,rb,rc,rd,re
end
function THPlaceableColorable.hook_registerSavegameXMLPaths(superFunc, schema, baseKey, ...)
local ra,rb,rc,rd,re = superFunc(schema, baseKey, ...)
local function appendFunc()
schema:setXMLSpecializationType("Colorable")
local oldBaseKey = baseKey
baseKey = oldBaseKey:gsub("%.colorable.*", "")
for i = 1, THPlaceableColorable.MAX_NUM_COLOR_SCALES do
local colorScaleKey = string.format("%s.placeableColors.colorScale%d", baseKey, i-1)
schema:register(XMLValueType.INT, colorScaleKey.."#color", "Active color index")
end
schema:setXMLSpecializationType()
end
g_thMain:pcall(appendFunc)
return ra,rb,rc,rd,re
end
function THPlaceableColorable:hook_onLoad(superFunc, savegame, ...)
local function prependFunc()
local specData = g_thMain:createDataTable(self.spec_colorable, true)
if specData then
specData.nodes = {}
specData.colors = {}
specData.colorIndex = {}
specData.colorScaleIndex = 0
specData.hasExtendedColors = false
for i = 1, THPlaceableColorable.MAX_NUM_COLOR_SCALES do
specData.nodes[i] = {}
specData.colors[i] = {}
specData.colorIndex[i] = 0
end
specData.setAllColors = function()
return THPlaceableColorable.setAllColors(self, specData)
end
specData.getAvailableColorsAtIndex = function(_, pIndex)
return THPlaceableColorable.getAvailableColorsAtIndex(self, pIndex, specData)
end
local xmlFile = self.xmlFile
local baseKey = "placeable.placeableColors"
local addedNodes = {}
local sharedColors = {}
if xmlFile:hasProperty(baseKey..".sharedColors") then
xmlFile:iterate(baseKey..".sharedColors.color", function(_, pColorKey)
local vColor = xmlFile:getValue(pColorKey.."#value", nil, true)
if vColor then
table.insert(sharedColors, vColor)
end
if #sharedColors == 255 then
return false
end
end)
end
for i = 1, THPlaceableColorable.MAX_NUM_COLOR_SCALES do
local colorScaleKey = string.format("%s.colorScale%d", baseKey, i-1)
local nodesKey = colorScaleKey..".nodes"
local nodesList = specData.nodes[i]
if xmlFile:hasProperty(nodesKey) then
xmlFile:iterate(nodesKey..".node", function(_, pNodeKey)
local vNodeId = xmlFile:getValue(pNodeKey.."#index", nil, self.components, self.i3dMappings)
if Utils.getNoNil(vNodeId, 0) ~= 0 then
local vShaderParamName = string.format("colorScale%d", i-1)
if not getHasShaderParameter(vNodeId, vShaderParamName) then
g_thMain:warningMsg("Node %q has no shader parameter %q for key %q!", getName(vNodeId), vShaderParamName, pNodeKey)
else
local vNodeColorScaleIndex = addedNodes[vNodeId]
if vNodeColorScaleIndex then
g_thMain:warningMsg("Node %q is already added to colorScale%d and will be ignored", getName(vNodeId), vNodeColorScaleIndex)
else
table.insert(nodesList, vNodeId)
addedNodes[vNodeId] = i-1
end
end
end
end)
end
if #nodesList > 0 then
local colorsKey = colorScaleKey..".colors"
local colorsList = specData.colors[i]
local useSharedColors = xmlFile:getValue(colorsKey.."#useShared", true)
if useSharedColors then
for _, vSharedColor in ipairs(sharedColors) do
table.insert(colorsList, vSharedColor)
end
end
if xmlFile:hasProperty(colorsKey) then
xmlFile:iterate(colorsKey..".color", function(_, pColorKey)
if #colorsList == 255 then
return false
end
local vColor = xmlFile:getValue(pColorKey.."#value", nil, true)
if vColor then
table.insert(colorsList, vColor)
end
end)
end
if #colorsList > 0 then
specData.colorIndex[i] = 1
specData.hasExtendedColors = true
if specData.colorScaleIndex <= 0 then
specData.colorScaleIndex = i
end
end
end
end
end
end
g_thMain:pcall(prependFunc)
return superFunc(self, savegame, ...)
end
function THPlaceableColorable:hook_setColor(superFunc, index, ...)
local _, specData = g_thMain:pcall("getDataTable", self.spec_colorable)
local realIndex = index
local function prependFunc()
if specData.hasExtendedColors then
index = 0
end
end
g_thMain:pcall(prependFunc)
local ra,rb,rc,rd,re = superFunc(self, index, ...)
local function appendFunc()
if specData.hasExtendedColors and realIndex and realIndex > 0 then
local colorScaleIndex = specData.colorScaleIndex
local colorsList = specData.colors[colorScaleIndex]
local colorValue = colorsList[realIndex]
if colorValue then
local shaderParam = string.format("colorScale%d", colorScaleIndex - 1)
local r,g,b = g_thMain:unpack(colorValue)
for _, nodeId in pairs(specData.nodes[colorScaleIndex]) do
setShaderParameter(nodeId, shaderParam, r,g,b, 1, false)
end
specData.colorIndex[colorScaleIndex] = realIndex
end
end
end
g_thMain:pcall(appendFunc)
return ra,rb,rc,rd,re
end
function THPlaceableColorable:hook_getHasColors(superFunc, ...)
local hasColors, ra,rb,rc,rd,re = superFunc(self, ...)
local function appendFunc()
local specData = g_thMain:getDataTable(self.spec_colorable)
if specData.hasExtendedColors then
hasColors = #specData.nodes[specData.colorScaleIndex] > 0 and #specData.colors[specData.colorScaleIndex] > 0
end
end
g_thMain:pcall(appendFunc)
return hasColors, ra,rb,rc,rd,re
end
function THPlaceableColorable:hook_getAvailableColors(superFunc, ...)
local colorsList, ra,rb,rc,rd,re = superFunc(self, ...)
local function appendFunc()
local specData = g_thMain:getDataTable(self.spec_colorable)
if specData.hasExtendedColors then
colorsList = specData.colors[specData.colorScaleIndex]
end
end
g_thMain:pcall(appendFunc)
return colorsList, ra,rb,rc,rd,re
end
function THPlaceableColorable:hook_getColor(superFunc, ...)
local colorIndex, ra,rb,rc,rd,re = superFunc(self, ...)
local function appendFunc()
local specData = g_thMain:getDataTable(self.spec_colorable)
if specData.hasExtendedColors then
colorIndex = specData.colorIndex[specData.colorScaleIndex]
end
end
g_thMain:pcall(appendFunc)
return colorIndex, ra,rb,rc,rd,re
end
function THPlaceableColorable:hook_onReadStream(superFunc, streamId, connection, ...)
local ra,rb,rc,rd,re = superFunc(self, streamId, connection)
local specData = g_thMain:getDataTable(self.spec_colorable)
if specData.hasExtendedColors and streamReadBool(streamId) then
specData.colorScaleIndex = streamReadUInt8(streamId)
for i = 1, THPlaceableColorable.MAX_NUM_COLOR_SCALES do
specData.colorIndex[i] = streamReadUInt8(streamId)
end
specData:setAllColors()
end
return ra,rb,rc,rd,re
end
function THPlaceableColorable:hook_onWriteStream(superFunc, streamId, connection, ...)
local ra,rb,rc,rd,re = superFunc(self, streamId, connection)
local specData = g_thMain:getDataTable(self.spec_colorable)
if streamWriteBool(streamId, specData.hasExtendedColors == true) then
streamWriteUInt8(streamId, specData.colorScaleIndex)
for i = 1, THPlaceableColorable.MAX_NUM_COLOR_SCALES do
streamWriteUInt8(streamId, specData.colorIndex[i])
end
end
return ra,rb,rc,rd,re
end
function THPlaceableColorable:hook_loadFromXMLFile(superFunc, xmlFile, key, ...)
local ra,rb,rc,rd,re = superFunc(self, xmlFile, key, ...)
local function appendFunc()
local specData = g_thMain:getDataTable(self.spec_colorable)
if specData.hasExtendedColors then
local baseKey = key:gsub("%.colorable.*", "")
for i = 1, THPlaceableColorable.MAX_NUM_COLOR_SCALES do
local colorScaleKey = string.format("%s.placeableColors.colorScale%d", baseKey, i-1)
if xmlFile:hasProperty(colorScaleKey) then
local colorsList = specData.colors[i]
local colorIndex = xmlFile:getValue(colorScaleKey.."#color", 0)
if colorIndex and colorIndex > 0 and #colorsList > 0 then
colorIndex = math.min(colorIndex, #colorsList)
specData.colorIndex[i] = colorIndex
end
end
end
specData:setAllColors()
end
end
g_thMain:pcall(appendFunc)
return ra,rb,rc,rd,re
end
function THPlaceableColorable:hook_saveToXMLFile(superFunc, xmlFile, key, useModNames, ...)
local _, specData, specInfo = g_thMain:pcall("getDataTable", self.spec_colorable)
local getHasColors_old = false
local function prependFunc()
if specData.hasExtendedColors then
local getHasColorsFunc = self.getHasColors
getHasColors_old = rawget(self, "getHasColors")
rawset(self, "getHasColors", function(pSelf, ...)
getHasColorsFunc(pSelf, ...)
return false
end)
end
end
g_thMain:pcall(prependFunc)
local ra,rb,rc,rd,re = superFunc(self, xmlFile, key, useModNames, ...)
if getHasColors_old ~= false then rawset(self, "getHasColors", getHasColors_old) end
local function appendFunc()
if specData.hasExtendedColors then
local baseKey = key:gsub("%.colorable.*", "")
for i = 1, THPlaceableColorable.MAX_NUM_COLOR_SCALES do
local colorScaleKey = string.format("%s.placeableColors.colorScale%d", baseKey, i-1)
local colorIndex = specData.colorIndex[i]
if colorIndex > 0 then
xmlFile:setValue(colorScaleKey.."#color", colorIndex)
end
end
end
end
g_thMain:pcall(appendFunc)
return ra,rb,rc,rd,re
end
g_thMain:hookFunction(PlaceableColorable, "registerXMLPaths",         THPlaceableColorable, nil, false)
g_thMain:hookFunction(PlaceableColorable, "registerSavegameXMLPaths", THPlaceableColorable, nil, false)
g_thMain:hookFunction(PlaceableColorable, "onLoad",                   THPlaceableColorable)
g_thMain:hookFunction(PlaceableColorable, "setColor",                 THPlaceableColorable)
g_thMain:hookFunction(PlaceableColorable, "getHasColors",             THPlaceableColorable)
g_thMain:hookFunction(PlaceableColorable, "getAvailableColors",       THPlaceableColorable)
g_thMain:hookFunction(PlaceableColorable, "getColor",                 THPlaceableColorable)
g_thMain:hookFunction(PlaceableColorable, "onReadStream",             THPlaceableColorable)
g_thMain:hookFunction(PlaceableColorable, "onWriteStream",            THPlaceableColorable)
g_thMain:hookFunction(PlaceableColorable, "loadFromXMLFile",          THPlaceableColorable)
g_thMain:hookFunction(PlaceableColorable, "saveToXMLFile",            THPlaceableColorable)